#include <extdll.h>
#include <sdk_util.h>
#include <meta_api.h>

#include "plugin.h"

static unsigned int GetPlayerWONId(edict_t * e) {
	//Old GameDLL.. tries to use WON on Steam server..
	// Steam engine returns 0xFFFFFFFF
	// We return "1222222200" + ent-index
	
	const unsigned int head = 1222222200;
	
	RETURN_META_VALUE(MRES_SUPERCEDE, head + (unsigned int)ENTINDEX(e));
}

static char * FixWON2Steam(char * in) {
	const int find_len = 9; //strlen of "<12222222"
	char * find;
	int in_len;
	int ent_index;
	edict_t * pent;
	const char * authid;
	int authid_len;
	char * newstr;
	char * endcheck;
	
	find = strstr(in, "<12222222");
	if(!find)
		return(0);
	
	if(*(find+find_len+0) < '0' || *(find+find_len+0) > '3')
		return(0);
	
	if(*(find+find_len+1) < '0' || *(find+find_len+1) > '9')
		return(0);
	
	if(*(find+find_len+2) != '>')
		return(0);
	
	//make number
	ent_index = (*(find+find_len+0) - '0') * 10 + (*(find+find_len+1) - '0');
	if(ent_index < 1 || ent_index > gpGlobals->maxClients)
		return(0);
	
	pent = INDEXENT(ent_index);
	if(!pent || pent->free)
		return(0);
	
	//get steam id
	authid = (*g_engfuncs.pfnGetPlayerAuthId)(pent);
	if(!authid)
		return(0);
	
	in_len = strlen(in);
	authid_len = strlen(authid);
	
	//check if ending has another wonid
	endcheck = FixWON2Steam(find+find_len+2);
	if(endcheck)
		in_len = in_len - strlen(find+find_len+2) + strlen(endcheck);
	
	newstr = (char*)malloc(in_len - find_len + authid_len + 1);
	if(!newstr) {
		free(endcheck);
		return(0);
	}
	
	//copy begining of string
	memcpy(newstr, in, (unsigned long)find - (unsigned long)in);
	newstr[(unsigned long)find - (unsigned long)in] = 0;
	
	//copy steamid
	strcat(newstr, "<");
	strcat(newstr, authid);
	
	if(endcheck) {
		//copy end of string
		strcat(newstr, endcheck);
		free(endcheck);
	}
	else {
		//copy end of string
		strcat(newstr, find+find_len+2);
	}
		
	return(newstr);
}

static void AlertMessage(ALERT_TYPE atype, char *szFmt, ...) {
	static char buf[1024];
	char * str;
	char * fixstr;
	
	va_list ap;
	va_start(ap, szFmt);
	
	if(!strcmp(szFmt, "%s")) {
		// Metamod always gives szFmt=="%s"
		// Just get first vararg as pointer
		str = va_arg(ap, char *);
	}
	else {
		// fallback
		buf[0] = 0;
		vsnprintf(buf, sizeof(buf), szFmt, ap);
		str = buf;
	}
	
	va_end(ap);
	
	fixstr = FixWON2Steam(str);
	if(!fixstr)
		RETURN_META(MRES_IGNORED);
	
	ALERT(atype, "%s", fixstr);
	
	free(fixstr);
	RETURN_META(MRES_SUPERCEDE);
}

int plugin_GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion) {
	memset(pengfuncsFromEngine, 0, sizeof(enginefuncs_t));
	
	pengfuncsFromEngine->pfnAlertMessage = AlertMessage;
	pengfuncsFromEngine->pfnGetPlayerWONId = GetPlayerWONId;
	
	return(TRUE);
}
